@@ -1,3 +1,5 @@
+
+
 #########################################################################
 #                                                                       #
 #                                                                       #
@@ -13,6 +15,7 @@
 import sys, os
 from calibre.ebooks.rtf2xml import copy, override_table, list_table
 from calibre.ptempfile import better_mktemp
+from . import open_for_read, open_for_write
 
 
 class PreambleDiv:
@@ -411,7 +414,7 @@ cw<ci<font-style<nu<0
         info = line[6:16]
         changed = self.__margin_dict.get(info)
         if changed is None:
-            print 'woops!'
+            print('woops!')
         else:
             self.__page[changed] = line[20:-1]
         # cw<pa<margin-lef<nu<1728
@@ -561,8 +564,8 @@ cw<ci<font-style<nu<0
 
     def make_preamble_divisions(self):
         self.__initiate_values()
-        read_obj = open(self.__file, 'r')
-        self.__write_obj = open(self.__write_to, 'w')
+        read_obj = open_for_read(self.__file)
+        self.__write_obj = open_for_write(self.__write_to)
         line_to_read = 1
         while line_to_read:
             line_to_read = read_obj.readline()
@@ -576,7 +579,7 @@ cw<ci<font-style<nu<0
                 self.__ob_group -= 1
             action = self.__state_dict.get(self.__state)
             if action is None:
-                print self.__state
+                print(self.__state)
             action(line)
         read_obj.close()
         self.__write_obj.close()
