@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -10,7 +9,7 @@ __docformat__ = 'restructuredtext en'
 import os, weakref, shutil, textwrap
 from collections import OrderedDict
 from functools import partial
-from future_builtins import map
+from polyglot.builtins import iteritems, itervalues, map, unicode_type
 
 from PyQt5.Qt import (QDialog, QGridLayout, QIcon, QCheckBox, QLabel, QFrame,
                       QApplication, QDialogButtonBox, Qt, QSize, QSpacerItem,
@@ -24,6 +23,7 @@ from calibre.gui2.dialogs.progress import ProgressDialog
 from calibre.ptempfile import PersistentTemporaryDirectory
 from calibre.utils.config_base import tweaks
 
+
 class Polish(QDialog):  # {{{
 
     def __init__(self, db, book_id_map, parent=None):
@@ -53,21 +53,24 @@ class Polish(QDialog):  # {{{
 
             'metadata':_('<h3>Updating metadata</h3>'
                          '<p>This will update all metadata <i>except</i> the cover in the'
-                         ' ebook files to match the current metadata in the'
+                         ' e-book files to match the current metadata in the'
                          ' calibre library.</p>'
-                         ' <p>Note that most ebook'
+                         ' <p>Note that most e-book'
                          ' formats are not capable of supporting all the'
                          ' metadata in calibre.</p><p>There is a separate option to'
                          ' update the cover.</p>'),
-            'do_cover': _('<h3>Update cover</h3><p>Update the covers in the ebook files to match the'
+            'do_cover': _('<h3>Update cover</h3><p>Update the covers in the e-book files to match the'
                         ' current cover in the calibre library.</p>'
-                        '<p>If the ebook file does not have'
+                        '<p>If the e-book file does not have'
                         ' an identifiable cover, a new cover is inserted.</p>'
                         ),
-            'jacket':_('<h3>Book Jacket</h3>%s')%HELP['jacket'],
-            'remove_jacket':_('<h3>Remove Book Jacket</h3>%s')%HELP['remove_jacket'],
+            'jacket':_('<h3>Book jacket</h3>%s')%HELP['jacket'],
+            'remove_jacket':_('<h3>Remove book jacket</h3>%s')%HELP['remove_jacket'],
             'remove_unused_css':_('<h3>Remove unused CSS rules</h3>%s')%HELP['remove_unused_css'],
             'compress_images': _('<h3>Losslessly compress images</h3>%s') % HELP['compress_images'],
+            'add_soft_hyphens': _('<h3>Add soft-hyphens</h3>%s') % HELP['add_soft_hyphens'],
+            'remove_soft_hyphens': _('<h3>Remove soft-hyphens</h3>%s') % HELP['remove_soft_hyphens'],
+            'upgrade_book': _('<h3>Upgrade book internals</h3>%s') % HELP['upgrade_book'],
         }
 
         self.l = l = QGridLayout()
@@ -83,17 +86,21 @@ class Polish(QDialog):  # {{{
             ('smarten_punctuation', _('Smarten &punctuation')),
             ('metadata', _('Update &metadata in the book files')),
             ('do_cover', _('Update the &cover in the book files')),
-            ('jacket', _('Add/Replace metadata as a "book &jacket" page')),
+            ('jacket', _('Add/replace metadata as a "book &jacket" page')),
             ('remove_jacket', _('&Remove a previously inserted book jacket')),
             ('remove_unused_css', _('Remove &unused CSS rules from the book')),
-            ('compress_images', _('Losslessly compress images')),
+            ('compress_images', _('Losslessly &compress images')),
+            ('add_soft_hyphens', _('Add s&oft hyphens')),
+            ('remove_soft_hyphens', _('Remove soft hyphens')),
+            ('upgrade_book', _('&Upgrade book internals')),
         ])
         prefs = gprefs.get('polishing_settings', {})
-        for name, text in self.all_actions.iteritems():
+        for name, text in iteritems(self.all_actions):
             count += 1
             x = QCheckBox(text, self)
             x.setChecked(prefs.get(name, False))
-            x.stateChanged.connect(partial(self.option_toggled, name))
+            x.setObjectName(name)
+            connect_lambda(x.stateChanged, self, lambda self, state: self.option_toggled(self.sender().objectName(), state))
             l.addWidget(x, count, 0, 1, 1)
             setattr(self, 'opt_'+name, x)
             la = QLabel(' <a href="#%s">%s</a>'%(name, _('About')))
@@ -102,14 +109,14 @@ class Polish(QDialog):  # {{{
             l.addWidget(la, count, 1, 1, 1)
 
         count += 1
-        l.addItem(QSpacerItem(10, 10, vPolicy=QSizePolicy.Expanding), count, 1, 1, 2)
+        l.addItem(QSpacerItem(10, 10, vPolicy=QSizePolicy.Policy.Expanding), count, 1, 1, 2)
 
         la = self.help_label = QLabel('')
         self.help_link_activated('#polish')
         la.setWordWrap(True)
-        la.setTextFormat(Qt.RichText)
-        la.setFrameShape(QFrame.StyledPanel)
-        la.setAlignment(Qt.AlignLeft|Qt.AlignTop)
+        la.setTextFormat(Qt.TextFormat.RichText)
+        la.setFrameShape(QFrame.Shape.StyledPanel)
+        la.setAlignment(Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)
         la.setLineWidth(2)
         la.setStyleSheet('QLabel { margin-left: 75px }')
         l.addWidget(la, 0, 2, count+1, 1)
@@ -120,18 +127,18 @@ class Polish(QDialog):  # {{{
         sr.setToolTip(textwrap.fill(_('Show a report of all the actions performed'
                         ' after polishing is completed')))
         l.addWidget(sr, count+1, 0, 1, 1)
-        self.bb = bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+        self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
-        self.save_button = sb = bb.addButton(_('&Save Settings'), bb.ActionRole)
+        self.save_button = sb = bb.addButton(_('&Save settings'), QDialogButtonBox.ButtonRole.ActionRole)
         sb.clicked.connect(self.save_settings)
-        self.load_button = lb = bb.addButton(_('&Load Settings'), bb.ActionRole)
+        self.load_button = lb = bb.addButton(_('&Load settings'), QDialogButtonBox.ButtonRole.ActionRole)
         self.load_menu = QMenu(lb)
         lb.setMenu(self.load_menu)
-        self.all_button = b = bb.addButton(_('Select &all'), bb.ActionRole)
-        b.clicked.connect(partial(self.select_all, True))
-        self.none_button = b = bb.addButton(_('Select &none'), bb.ActionRole)
-        b.clicked.connect(partial(self.select_all, False))
+        self.all_button = b = bb.addButton(_('Select &all'), QDialogButtonBox.ButtonRole.ActionRole)
+        connect_lambda(b.clicked, self, lambda self: self.select_all(True))
+        self.none_button = b = bb.addButton(_('Select &none'), QDialogButtonBox.ButtonRole.ActionRole)
+        connect_lambda(b.clicked, self, lambda self: self.select_all(False))
         l.addWidget(bb, count+1, 1, 1, -1)
         self.setup_load_button()
 
@@ -152,7 +159,7 @@ class Polish(QDialog):  # {{{
         name, ok = QInputDialog.getText(self, _('Choose name'),
                 _('Choose a name for these settings'))
         if ok:
-            name = unicode(name).strip()
+            name = unicode_type(name).strip()
             if name:
                 settings = {ac:getattr(self, 'opt_'+ac).isChecked() for ac in
                             self.all_actions}
@@ -187,11 +194,11 @@ class Polish(QDialog):  # {{{
             x.blockSignals(False)
 
     def option_toggled(self, name, state):
-        if state == Qt.Checked:
+        if state == Qt.CheckState.Checked:
             self.help_label.setText(self.help_text[name])
 
     def help_link_activated(self, link):
-        link = unicode(link)[1:]
+        link = unicode_type(link)[1:]
         self.help_label.setText(self.help_text[link])
 
     @property
@@ -237,7 +244,7 @@ class Polish(QDialog):  # {{{
         self.tdir = PersistentTemporaryDirectory('_queue_polish')
         self.jobs = []
         if len(self.book_id_map) <= 5:
-            for i, (book_id, formats) in enumerate(self.book_id_map.iteritems()):
+            for i, (book_id, formats) in enumerate(iteritems(self.book_id_map)):
                 self.do_book(i+1, book_id, formats)
         else:
             self.queue = [(i+1, id_) for i, id_ in enumerate(self.book_id_map)]
@@ -254,7 +261,7 @@ class Polish(QDialog):  # {{{
             self.jobs = []
             self.pd.reject()
             return
-        num, book_id = self.queue.pop()
+        num, book_id = self.queue.pop(0)
         try:
             self.do_book(num, book_id, self.book_id_map[book_id])
         except:
@@ -265,7 +272,7 @@ class Polish(QDialog):  # {{{
             QTimer.singleShot(0, self.do_one)
 
     def do_book(self, num, book_id, formats):
-        base = os.path.join(self.tdir, unicode(book_id))
+        base = os.path.join(self.tdir, unicode_type(book_id))
         os.mkdir(base)
         db = self.db()
         opf = os.path.join(base, 'metadata.opf')
@@ -285,9 +292,13 @@ class Polish(QDialog):  # {{{
                 db.copy_format_to(book_id, fmt, f, index_is_id=True)
                 data['files'].append(f.name)
 
+        nums = num
+        if hasattr(self, 'pd'):
+            nums = self.pd.max - num
+
         desc = ngettext(_('Polish %s')%mi.title,
                         _('Polish book %(nums)s of %(tot)s (%(title)s)')%dict(
-                            nums=num, tot=len(self.book_id_map),
+                            nums=nums, tot=len(self.book_id_map),
                             title=mi.title), len(self.book_id_map))
         if hasattr(self, 'pd'):
             self.pd.set_msg(_('Queueing book %(nums)s of %(tot)s (%(title)s)')%dict(
@@ -296,12 +307,13 @@ class Polish(QDialog):  # {{{
         self.jobs.append((desc, data, book_id, base, is_orig))
 # }}}
 
+
 class Report(QDialog):  # {{{
 
     def __init__(self, parent):
         QDialog.__init__(self, parent)
         self.gui = parent
-        self.setAttribute(Qt.WA_DeleteOnClose, False)
+        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, False)
         self.setWindowIcon(QIcon(I('polish.png')))
         self.reports = []
 
@@ -316,24 +328,24 @@ class Report(QDialog):  # {{{
         la.setVisible(False)
         la.setWordWrap(True)
 
-        self.ign_msg = _('Ignore remaining %d reports')
-        self.ign = QCheckBox(self.ign_msg, self)
+        self.ign = QCheckBox(_('Ignore remaining reports'), self)
         l.addWidget(self.ign, 2, 0)
 
-        bb = self.bb = QDialogButtonBox(QDialogButtonBox.Close)
+        bb = self.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Close)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
-        b = self.log_button = bb.addButton(_('View full &log'), bb.ActionRole)
+        b = self.log_button = bb.addButton(_('View full &log'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.view_log)
-        bb.button(bb.Close).setDefault(True)
+        bb.button(QDialogButtonBox.StandardButton.Close).setDefault(True)
         l.addWidget(bb, 2, 1)
 
-        self.finished.connect(self.show_next, type=Qt.QueuedConnection)
+        self.finished.connect(self.show_next, type=Qt.ConnectionType.QueuedConnection)
 
         self.resize(QSize(800, 600))
 
     def setup_ign(self):
-        self.ign.setText(self.ign_msg%len(self.reports))
+        self.ign.setText(ngettext(
+            'Ignore remaining report', 'Ignore remaining {} reports', len(self.reports)).format(len(self.reports)))
         self.ign.setVisible(bool(self.reports))
         self.ign.setChecked(False)
 
@@ -349,7 +361,7 @@ class Report(QDialog):  # {{{
         self.setWindowTitle(_('Polishing of %s')%book_title)
         self.view.setText(markdown('# %s\n\n'%book_title + report,
                                    output_format='html4'))
-        self.bb.button(self.bb.Close).setFocus(Qt.OtherFocusReason)
+        self.bb.button(QDialogButtonBox.StandardButton.Close).setFocus(Qt.FocusReason.OtherFocusReason)
         self.backup_msg.setVisible(bool(fmts))
         if fmts:
             m = ngettext('The original file has been saved as %s.',
@@ -389,6 +401,7 @@ class Report(QDialog):  # {{{
         super(Report, self).reject()
 # }}}
 
+
 class PolishAction(InterfaceAction):
 
     name = 'Polish Books'
@@ -411,7 +424,7 @@ class PolishAction(InterfaceAction):
     def drop_event(self, event, mime_data):
         mime = 'application/calibre+from_library'
         if mime_data.hasFormat(mime):
-            self.dropped_ids = tuple(map(int, str(mime_data.data(mime)).split()))
+            self.dropped_ids = tuple(map(int, mime_data.data(mime).data().split()))
             QTimer.singleShot(1, self.do_drop)
             return True
         return False
@@ -425,10 +438,19 @@ class PolishAction(InterfaceAction):
     def genesis(self):
         self.qaction.triggered.connect(self.polish_books)
         self.report = Report(self.gui)
+        self.to_be_refreshed = set()
+        self.refresh_debounce_timer = t = QTimer(self.gui)
+        t.setSingleShot(True)
+        t.setInterval(1000)
+        t.timeout.connect(self.refresh_after_polish)
+
+    def shutting_down(self):
+        self.refresh_debounce_timer.stop()
 
     def location_selected(self, loc):
         enabled = loc == 'library'
         self.qaction.setEnabled(enabled)
+        self.menuless_qaction.setEnabled(enabled)
 
     def get_books_for_polishing(self):
         rows = [r.row() for r in
@@ -441,7 +463,7 @@ class PolishAction(InterfaceAction):
         db = self.gui.library_view.model().db
         ans = (db.id(r) for r in rows)
         ans = self.get_supported_books(ans)
-        for fmts in ans.itervalues():
+        for fmts in itervalues(ans):
             for x in fmts:
                 if x.startswith('ORIGINAL_'):
                     from calibre.gui2.dialogs.confirm_delete import confirm
@@ -470,7 +492,7 @@ class PolishAction(InterfaceAction):
                   ' formats. Convert to one of those formats before polishing.')
                          %_(' or ').join(sorted(SUPPORTED)), show=True)
         ans = OrderedDict(ans)
-        for fmts in ans.itervalues():
+        for fmts in itervalues(ans):
             for x in SUPPORTED:
                 if ('ORIGINAL_'+x) in fmts:
                     fmts.discard(x)
@@ -484,7 +506,7 @@ class PolishAction(InterfaceAction):
 
     def do_polish(self, book_id_map):
         d = Polish(self.gui.library_view.model().db, book_id_map, parent=self.gui)
-        if d.exec_() == d.Accepted and d.jobs:
+        if d.exec_() == QDialog.DialogCode.Accepted and d.jobs:
             show_reports = bool(d.show_reports.isChecked())
             for desc, data, book_id, base, is_orig in reversed(d.jobs):
                 job = self.gui.job_manager.run_job(
@@ -494,7 +516,8 @@ class PolishAction(InterfaceAction):
             if d.jobs:
                 self.gui.jobs_pointer.start()
                 self.gui.status_bar.show_message(
-                    _('Start polishing of %d book(s)') % len(d.jobs), 2000)
+                    ngettext('Start polishing the book', 'Start polishing of {} books',
+                             len(d.jobs)).format(len(d.jobs)), 2000)
 
     def book_polished(self, job):
         if job.failed:
@@ -517,13 +540,22 @@ class PolishAction(InterfaceAction):
             os.rmdir(parent)
         except:
             pass
-        self.gui.tags_view.recount()
+        self.to_be_refreshed.add(book_id)
+        self.refresh_debounce_timer.start()
+        if show_reports:
+            self.report(db.title(book_id, index_is_id=True), book_id, fmts, job, job.result)
+
+    def refresh_after_polish(self):
+        self.refresh_debounce_timer.stop()
+        book_ids = tuple(self.to_be_refreshed)
+        self.to_be_refreshed = set()
         if self.gui.current_view() is self.gui.library_view:
+            self.gui.library_view.model().refresh_ids(book_ids)
             current = self.gui.library_view.currentIndex()
             if current.isValid():
                 self.gui.library_view.model().current_changed(current, QModelIndex())
-        if show_reports:
-            self.report(db.title(book_id, index_is_id=True), book_id, fmts, job, job.result)
+        self.gui.tags_view.recount()
+
 
 if __name__ == '__main__':
     app = QApplication([])
@@ -531,4 +563,3 @@ if __name__ == '__main__':
     from calibre.library import db
     d = Polish(db(), {1:{'EPUB'}, 2:{'AZW3'}})
     d.exec_()
-
