@@ -9,7 +9,7 @@ import json, os, traceback
 
 from PyQt5.Qt import (Qt, QDialog, QDialogButtonBox, QSyntaxHighlighter, QFont,
                       QRegExp, QApplication, QTextCharFormat, QColor, QCursor,
-                      QIcon, QSize)
+                      QIcon, QSize, QPalette)
 
 from calibre import sanitize_file_name
 from calibre.constants import config_dir
@@ -22,7 +22,7 @@ from calibre.ebooks.metadata.book.formatter import SafeFormat
 from calibre.library.coloring import (displayable_columns, color_row_key)
 from calibre.gui2 import error_dialog, choose_files, pixmap_to_data
 from calibre.utils.localization import localize_user_manual_link
-from polyglot.builtins import native_string_type, unicode_type
+from polyglot.builtins import unicode_type
 
 
 class ParenPosition:
@@ -44,9 +44,9 @@ class TemplateHighlighter(QSyntaxHighlighter):
     Formats = {}
     BN_FACTOR = 1000
 
-    KEYWORDS = ["program", 'if', 'then', 'else', 'fi']
+    KEYWORDS = ["program", 'if', 'then', 'else', 'elif', 'fi']
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, builtin_functions=None):
         super(TemplateHighlighter, self).__init__(parent)
 
         self.initializeFormats()
@@ -56,7 +56,8 @@ class TemplateHighlighter(QSyntaxHighlighter):
                 "keyword"))
         TemplateHighlighter.Rules.append((QRegExp(
                 "|".join([r"\b%s\b" % builtin for builtin in
-                          formatter_functions().get_builtins()])),
+                          (builtin_functions if builtin_functions else
+                                                formatter_functions().get_builtins())])),
                 "builtin"))
 
         TemplateHighlighter.Rules.append((QRegExp(
@@ -85,8 +86,8 @@ class TemplateHighlighter(QSyntaxHighlighter):
         pal = QApplication.instance().palette()
         for name, color, bold, italic in (
                 ("normal", None, False, False),
-                ("keyword", pal.color(pal.Link).name(), True, False),
-                ("builtin", pal.color(pal.Link).name(), False, False),
+                ("keyword", pal.color(QPalette.ColorRole.Link).name(), True, False),
+                ("builtin", pal.color(QPalette.ColorRole.Link).name(), False, False),
                 ("comment", "#007F00", False, True),
                 ("string", "#808000", False, False),
                 ("number", "#924900", False, False),
@@ -107,7 +108,7 @@ class TemplateHighlighter(QSyntaxHighlighter):
             if col:
                 format.setForeground(QColor(col))
             if Config["%sfontbold" % name]:
-                format.setFontWeight(QFont.Bold)
+                format.setFontWeight(QFont.Weight.Bold)
             format.setFontItalic(Config["%sfontitalic" % name])
             self.Formats[name] = format
 
@@ -166,7 +167,7 @@ class TemplateHighlighter(QSyntaxHighlighter):
                 i += 1
 
     def rehighlight(self):
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
+        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
         QSyntaxHighlighter.rehighlight(self)
         QApplication.restoreOverrideCursor()
 
@@ -213,7 +214,8 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
 
     def __init__(self, parent, text, mi=None, fm=None, color_field=None,
                  icon_field_key=None, icon_rule_kind=None, doing_emblem=False,
-                 text_is_placeholder=False):
+                 text_is_placeholder=False, dialog_is_st_editor=False,
+                 global_vars={}, all_functions=None, builtin_functions=None):
         QDialog.__init__(self, parent)
         Ui_TemplateDialog.__init__(self)
         self.setupUi(self)
@@ -221,6 +223,8 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
         self.coloring = color_field is not None
         self.iconing = icon_field_key is not None
         self.embleming = doing_emblem
+        self.dialog_is_st_editor = dialog_is_st_editor
+        self.global_vars = global_vars
 
         cols = []
         if fm is not None:
@@ -239,7 +243,8 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
         if self.coloring:
             self.color_layout.setVisible(True)
             for n1, k1 in cols:
-                self.colored_field.addItem(n1, k1)
+                self.colored_field.addItem(n1 +
+                       (' (' + k1 + ')' if k1 != color_row_key else ''), k1)
             self.colored_field.setCurrentIndex(self.colored_field.findData(color_field))
         elif self.iconing or self.embleming:
             self.icon_layout.setVisible(True)
@@ -250,7 +255,7 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
                 self.icon_field.setVisible(False)
 
             for n1, k1 in cols:
-                self.icon_field.addItem(n1, k1)
+                self.icon_field.addItem('{} ({})'.format(n1, k1), k1)
             self.icon_file_names = []
             d = os.path.join(config_dir, 'cc_icons')
             if os.path.exists(d):
@@ -273,6 +278,14 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
                 self.icon_kind.setCurrentIndex(dex)
                 self.icon_field.setCurrentIndex(self.icon_field.findData(icon_field_key))
 
+        if dialog_is_st_editor:
+            self.buttonBox.setVisible(False)
+        else:
+            self.new_doc_label.setVisible(False)
+            self.new_doc.setVisible(False)
+            self.template_name_label.setVisible(False)
+            self.template_name.setVisible(False)
+
         if mi:
             self.mi = mi
         else:
@@ -294,14 +307,19 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
                 from calibre.gui2.ui import get_gui
                 self.mi.set_all_user_metadata(
                       get_gui().current_db.new_api.field_metadata.custom_field_metadata())
+            for col in self.mi.get_all_user_metadata(False):
+                self.mi.set(col, (col,), 0)
 
         # Remove help icon on title bar
         icon = self.windowIcon()
-        self.setWindowFlags(self.windowFlags()&(~Qt.WindowContextHelpButtonHint))
+        self.setWindowFlags(self.windowFlags()&(~Qt.WindowType.WindowContextHelpButtonHint))
         self.setWindowIcon(icon)
 
+        self.all_functions = all_functions if all_functions else formatter_functions().get_functions()
+        self.builtins = builtin_functions if builtin_functions else formatter_functions().get_builtins()
+
         self.last_text = ''
-        self.highlighter = TemplateHighlighter(self.textbox.document())
+        self.highlighter = TemplateHighlighter(self.textbox.document(), builtin_functions=self.builtins)
         self.textbox.cursorPositionChanged.connect(self.text_cursor_changed)
         self.textbox.textChanged.connect(self.textbox_changed)
 
@@ -313,10 +331,11 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
         if text is not None:
             if text_is_placeholder:
                 self.textbox.setPlaceholderText(text)
+                self.textbox.clear()
             else:
                 self.textbox.setPlainText(text)
-        self.buttonBox.button(QDialogButtonBox.Ok).setText(_('&OK'))
-        self.buttonBox.button(QDialogButtonBox.Cancel).setText(_('&Cancel'))
+        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setText(_('&OK'))
+        self.buttonBox.button(QDialogButtonBox.StandardButton.Cancel).setText(_('&Cancel'))
         self.color_copy_button.clicked.connect(self.color_to_clipboard)
         self.filename_button.clicked.connect(self.filename_button_clicked)
         self.icon_copy_button.clicked.connect(self.icon_to_clipboard)
@@ -327,15 +346,14 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
         except:
             self.builtin_source_dict = {}
 
-        self.funcs = formatter_functions().get_functions()
-        self.builtins = formatter_functions().get_builtins()
-
-        func_names = sorted(self.funcs)
+        func_names = sorted(self.all_functions)
         self.function.clear()
         self.function.addItem('')
-        self.function.addItems(func_names)
+        for f in func_names:
+            self.function.addItem('{}  --  {}'.format(f,
+                               self.function_type_string(f, longform=False)), f)
         self.function.setCurrentIndex(0)
-        self.function.currentIndexChanged[native_string_type].connect(self.function_changed)
+        self.function.currentIndexChanged.connect(self.function_changed)
         self.textbox_changed()
         self.rule = (None, '')
 
@@ -412,8 +430,9 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
             self.highlighter.regenerate_paren_positions()
             self.text_cursor_changed()
             self.template_value.setText(
-                SafeFormat().safe_format(cur_text, self.mi,
-                                                _('EXCEPTION: '), self.mi))
+                SafeFormat().safe_format(cur_text, self.mi, _('EXCEPTION: '),
+                                         self.mi, global_vars=self.global_vars,
+                                         template_functions=self.all_functions))
 
     def text_cursor_changed(self):
         cursor = self.textbox.textCursor()
@@ -425,16 +444,28 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
             self.highlighter.check_cursor_pos(t[position-1], block_number,
                                               pos_in_block)
 
+    def function_type_string(self, name, longform=True):
+        if self.all_functions[name].is_python:
+            if name in self.builtins:
+                return (_('Built-in template function') if longform else
+                            _('Built-in function'))
+            return (_('User defined Python template function') if longform else
+                            _('User function'))
+        else:
+            return (_('Stored user defined template') if longform else _('Stored template'))
+
     def function_changed(self, toWhat):
-        name = unicode_type(toWhat)
+        name = unicode_type(self.function.itemData(toWhat))
         self.source_code.clear()
         self.documentation.clear()
-        if name in self.funcs:
-            self.documentation.setPlainText(self.funcs[name].doc)
+        self.func_type.clear()
+        if name in self.all_functions:
+            self.documentation.setPlainText(self.all_functions[name].doc)
             if name in self.builtins and name in self.builtin_source_dict:
                 self.source_code.setPlainText(self.builtin_source_dict[name])
             else:
-                self.source_code.setPlainText(self.funcs[name].program_text)
+                self.source_code.setPlainText(self.all_functions[name].program_text)
+            self.func_type.setText(self.function_type_string(name, longform=True))
 
     def accept(self):
         txt = unicode_type(self.textbox.toPlainText()).rstrip()
@@ -462,6 +493,27 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
             self.rule = ('', txt)
         QDialog.accept(self)
 
+    def reject(self):
+        QDialog.reject(self)
+        if self.dialog_is_st_editor:
+            parent = self.parent()
+            while True:
+                if hasattr(parent, 'reject'):
+                    parent.reject()
+                    break
+                parent = parent.parent()
+                if parent is None:
+                    break
+
+
+class EmbeddedTemplateDialog(TemplateDialog):
+
+    def __init__(self, parent):
+        TemplateDialog.__init__(self, parent, _('A General Program Mode Template'), text_is_placeholder=True,
+                                dialog_is_st_editor=True)
+        self.setParent(parent)
+        self.setWindowFlags(Qt.WindowType.Widget)
+
 
 if __name__ == '__main__':
     app = QApplication([])
